// Sketch to draw an analogue clock on the screen
// This uses anti-aliased drawing functions that are built into TFT_eSPI

// Anti-aliased lines can be drawn with sub-pixel resolution and permit lines to be
// drawn with less jaggedness.

// Based on a sketch by DavyLandman:
// https://github.com/Bodmer/TFT_eSPI/issues/905

#include <Arduino.h>
#include <TFT_eSPI.h>       // Master copy here: https://github.com/Bodmer/TFT_eSPI
#include "NotoSansBold15.h" // Zeichensatz
#include <WiFi.h>           // WLAN-Kram
#include "time.h"           // Zeitfunktionen inkl. NTP
#include "esp_sntp.h"       // Zugriff auf die NTP-Innereien in der ESP32 Umgebung

TFT_eSPI tft = TFT_eSPI();  // Invoke library, pins defined in User_Setup.h
TFT_eSprite face = TFT_eSprite(&tft);

#define CLOCK_X_POS 0
#define CLOCK_Y_POS 0

#define CLOCK_FG   TFT_SKYBLUE
#define CLOCK_BG   TFT_BLACK
#define SECCOND_FG TFT_RED
#define LABEL_FG   TFT_GOLD

#define CLOCK_R       230.0f / 2.0f // Passt für rundes 1,28 Zoll Display
#define H_HAND_LENGTH CLOCK_R/2.0f
#define M_HAND_LENGTH CLOCK_R/1.4f
#define S_HAND_LENGTH CLOCK_R/1.3f

#define FACE_W CLOCK_R * 2 + 1
#define FACE_H CLOCK_R * 2 + 1

// Calculate 1 second increment angles. Hours and minute hand angles
// change every second so we see smooth sub-pixel movement
#define SECOND_ANGLE 360.0 / 60.0
#define MINUTE_ANGLE SECOND_ANGLE / 60.0
#define HOUR_ANGLE   MINUTE_ANGLE / 12.0

// Sprite width and height
#define FACE_W CLOCK_R * 2 + 1
#define FACE_H CLOCK_R * 2 + 1
struct tm tm;
uint8_t h = 0, m = 0, s = 0, upd = 0;
float time_secs = h * 3600 + m * 60 + s;
const char* ssid     = "Router-SSID";           // SSID
const char* password = "Passwort";              // Passwort
const char* ntpServer = "ptbtime1.ptb.de";      // NTP-Server
// const char* ntpServer = "fritz.box";         // NTP-Server Fritzbox
const long  gmtOffset_sec = 3600;               // MEZ
const int   daylightOffset_sec = 3600;          // MESZ
uint32_t targetTime = 0;

// =========================================================================
// Draw the clock face in the sprite
// =========================================================================
static void renderFace(float t) {
  float h_angle = t * HOUR_ANGLE;
  float m_angle = t * MINUTE_ANGLE;
  float s_angle = t * SECOND_ANGLE;

  // The face is completely redrawn - this can be done quickly
  face.fillSprite(TFT_BLACK);

  // Draw the face circle
  face.fillSmoothCircle( CLOCK_R, CLOCK_R, CLOCK_R, CLOCK_BG );

  // Set text datum to middle centre and the colour
  face.setTextDatum(MC_DATUM);

  // The background colour will be read during the character rendering
  face.setTextColor(CLOCK_FG, CLOCK_BG);

  // Text offset adjustment
  constexpr uint32_t dialOffset = CLOCK_R - 10;

  float xp = 0.0, yp = 0.0; // Use float pixel position for smooth AA motion

  // Draw digits around clock perimeter
  for (uint32_t h = 1; h <= 12; h++) {
    getCoord(CLOCK_R, CLOCK_R, &xp, &yp, dialOffset, h * 360.0 / 12);
    face.drawNumber(h, xp, 2 + yp);
  }

  // Add text (could be digital time...)
  face.setTextColor(LABEL_FG, CLOCK_BG);
  face.drawString("katzenjens", CLOCK_R, CLOCK_R * 0.75); // Nach Wunsch veränderbar ;)

  // Draw minute hand
  getCoord(CLOCK_R, CLOCK_R, &xp, &yp, M_HAND_LENGTH, m_angle);
  face.drawWideLine(CLOCK_R, CLOCK_R, xp, yp, 6.0f, CLOCK_FG);
  face.drawWideLine(CLOCK_R, CLOCK_R, xp, yp, 2.0f, CLOCK_BG);

  // Draw hour hand
  getCoord(CLOCK_R, CLOCK_R, &xp, &yp, H_HAND_LENGTH, h_angle);
  face.drawWideLine(CLOCK_R, CLOCK_R, xp, yp, 6.0f, CLOCK_FG);
  face.drawWideLine(CLOCK_R, CLOCK_R, xp, yp, 2.0f, CLOCK_BG);

  // Draw the central pivot circle
  face.fillSmoothCircle(CLOCK_R, CLOCK_R, 4, CLOCK_FG);

  // Draw cecond hand
  getCoord(CLOCK_R, CLOCK_R, &xp, &yp, S_HAND_LENGTH, s_angle);
  face.drawWedgeLine(CLOCK_R, CLOCK_R, xp, yp, 2.5, 1.0, SECCOND_FG);
  face.pushSprite(5, 5, TFT_TRANSPARENT);
}

// =========================================================================
// Get coordinates of end of a line, pivot at x,y, length r, angle a
// =========================================================================
// Coordinates are returned to caller via the xp and yp pointers
#define DEG2RAD 0.0174532925
void getCoord(int16_t x, int16_t y, float *xp, float *yp, int16_t r, float a)
{
  float sx1 = cos( (a - 90) * DEG2RAD);
  float sy1 = sin( (a - 90) * DEG2RAD);
  *xp =  sx1 * r + x;
  *yp =  sy1 * r + y;
}
// -----------------------

void wifi_ein(){
 tft.setCursor(30, 40);
 tft.println("Connecting to ");
 tft.setCursor(30, 60);
 tft.println(ssid);
 tft.setCursor(30, 80);
 WiFi.begin(ssid, password);
 while (WiFi.status() != WL_CONNECTED) {
   delay(500);
   tft.print(".");
  }
 tft.setCursor(30, 100);
 tft.print("WiFi connected.");
}
// =========================================================================
void get_ntp(){
 // Init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  tft.setCursor(30, 120);
  tft.println("Get NTP time");
  tft.setCursor(30, 140);
  do {
      tft.print("."); // warten bis NTP synchronisiert ist
    }
  while (!getLocalTime(&tm));
  tft.setCursor(30,160);
  tft.println("NTP ok.");
  delay(1000);
  time_secs = tm.tm_hour * 3600 + tm.tm_min * 60 + tm.tm_sec; // Update the clock time
}
// =========================================================================
void syncTime(){
  getLocalTime(&tm);
  time_secs = tm.tm_hour * 3600 + tm.tm_min * 60 + tm.tm_sec; // Update the clock time
}
// =========================================================================
// Setup
// =========================================================================
void setup() {
  // Initialise the screen
  tft.init();
  tft.setRotation(0);
  tft.fillScreen(TFT_BLACK);
  tft.loadFont(NotoSansBold15);
  tft.setTextSize(1);
  wifi_ein(); // WLAN
  delay(1000);
  get_ntp(); // NTP über Netz
  delay(1000);
  tft.setRotation(0);
  tft.fillScreen(TFT_BLACK);
  sntp_set_sync_interval(3600000);      // 1 Stunde bis resync durchs Netz
  face.createSprite(FACE_W, FACE_H);    // Create the clock face sprite
  face.loadFont(NotoSansBold15);        // Only 1 font used in the sprite, so can remain loaded
}

// =========================================================================
// Loop
// =========================================================================
void loop() { // Schleichender Zeiger
  if (targetTime < millis()) {
    // ggfls. anpassen für "Smoothigkeit"
    targetTime = millis() + 98;
    // Increment time by 100 milliseconds
    time_secs += 0.100;
    // Midnight roll-over
    if (time_secs >= (60 * 60 * 24)) time_secs = 0;
    renderFace(time_secs);      
    upd++;            //sekündlich mit time abgleichen da millis zu ungenau.
    if (upd > 9){
      syncTime();
      upd = 0;
      return;
    }
  }
}

